# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2
from driverng import Worklist
from driverng.exceptions import ParsingError
from driverng.fields import String


class PvsBase(object):
    FIELDS = ()

    def __init__(self, **kwargs):
        for code, field in self.FIELDS:
            setattr(self, field.name, kwargs.get(field.name, None))
        super(PvsBase, self).__init__()

    def parse(self, data):
        fields = dict(self.FIELDS)
        for elt in data.split("|"):
            if not elt:
                continue
            code = elt[:3]
            try:
                field = fields[code]
            except KeyError:
                pass
            else:
                value = field.to_python(elt[4:])
                setattr(self, field.name, value)

    def to_string(self):
        values = []
        for code, field in self.FIELDS:
            values.append("{}:{}".format(
                code,
                field.to_string(getattr(self, field.name)),
            ))
        return "%s|" % ("|".join(values), )


class PvsOrder(PvsBase):
    FIELDS = (
        ("TYP", String("typ", 3)),
        ("SID", String("tube", 16)),
        # a vérifier la taille
        ("SYS", String("system", 16)),
        ("NAM", String("nom", 16)),
        ("MAT", String("material_code")),
        ("TST", String("tests", 64)),
    )


class PvsQuery(PvsBase):
    FIELDS = (
        ("TYP", String("typ", 3)),
        ("SID", String("tube", 16)),
        ("TST", String("tests", 64)),
        ("MAT", String("material", 16))
    )


class PvsResult(PvsBase):
    FIELDS = (
        ("TYP", String("typ", 3)),
        ("SID", String("tube", 16)),
        ("TRG", String("rack")),
        ("POS", String("pos", 8)),
    )


class PvsMultipleMessage(list):
    _MESSAGE_TYPE_MAP = {
        "RQ": PvsOrder,
        "RW": PvsOrder,
        "LA": PvsQuery,
        "WP": PvsResult,
    }

    def add(self, message_type=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[message_type](
            message_type=message_type, **kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if line:
                if line[:3] != "TYP":
                    raise ParsingError("Missing message type")
                obj = self._MESSAGE_TYPE_MAP[line[4:][:2]]()
                obj.parse(line)
                self.append(obj)

    def to_string(self):
        return "\n".join([
            e.to_string() for e in self
        ])


class Driver(MessageV2):
    SPEC = {
        "code": "pvs",
        "label": "Pvs Trieur Sarstedt",
    }

    MESSAGE_TYPE_MAP = {
        "RQ": PvsOrder,
        "RW": PvsOrder,
        "LA": PvsQuery,
        "WP": PvsResult,
    }

    COLOR_CODE = {
        "red": "ff0000",
        "blue": "0000ff",
        "gold": "ffd700",
        "black": "000000",
        "green": "008000",
        "orange": "ffa500",
        "grey": "808080",
        "yellow": "ffff00",
        "brown": "a52a2a",
        "pink": "ffc0cb",
        "white": "ffffff",
        "violet": "ee82ee",
        "purple": "800080",
        "beige": "fff5ee",
    }

    OPTIONS = [
        MessageV2.BooleanOption(
            "pvs_version_routage_zone",
            label="Routage du trieur avec envoi de la zone",
            default=False,
        ),
        MessageV2.BooleanOption(
            "pvs_verifier_couleur_bouchon",
            label="Vérifier la couleur du bouchon avec la couleur de la nature dans le cas de la version du trieur avec routage",
            default=False,
        )
    ]

    def __init__(
            self,
            pvs_version_routage_zone=False,
            pvs_verifier_couleur_bouchon=False,
            **kwargs
    ):
        self.pvs_version_routage_zone = pvs_version_routage_zone
        self.pvs_verifier_couleur_bouchon = pvs_verifier_couleur_bouchon
        return super(Driver, self).__init__(**kwargs)

    def create(self, message_type, **kwargs):
        return self.MESSAGE_TYPE_MAP[message_type](**kwargs)

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = PvsMultipleMessage()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = ""
        for sample, action, priority in worklist.get_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"A": "STAT", "S": "STAT", worklist.ELSE: ""},
        ):
            nom_patient = "{}_{}".format(sample.last_name, sample.first_name)
            message_type = "RQ"
            if self.pvs_version_routage_zone:
                for order in sample.orders:
                    if order["dem_auto_test_status"] == 1:
                        # un seul test avec le status 1 (chargé) suffit à
                        # l'envoi du type de message RW
                        message_type = "RW"
                        continue
                    else:
                        message_type = "RQ"
            msg = self.create(
                message_type,
                typ=message_type,
                tube=sample.sample_id,
                nom=nom_patient,
                material_code=sample.specimen
            )
            # liste des tests
            tests = ",".join([order["code"] for order in sample.orders])
            if self.pvs_version_routage_zone and not self.pvs_verifier_couleur_bouchon:
                if order["zone_groupement_test"]:
                    codes_zones = ",".join(set([order["zone_groupement_test"] for order in sample.orders]))
                    if priority:
                        msg.tests = "{},STAT".format(codes_zones)
                    else:
                        msg.tests = "{}".format(codes_zones)
                else:
                    msg.tests = ""
            elif self.pvs_version_routage_zone and self.pvs_verifier_couleur_bouchon:
                # Vérification de la couleur des tubes exceptés les tubes
                # débouchés (OPEN) et les tubes sans identification de la
                # couleur dans le query (tube débouché)
                if (
                        not context["query"][0].material or
                        (len(context["query"][0].material.split(" ")) == 4 and
                            (context["query"][0].material.split(" ")[3] == "OPEN" or order["nature_color"] == self.COLOR_CODE.get(context["query"][0].material.split(" ")[3])))
                ):
                    # Couleur compatible avec la couleur paramétrée dans le
                    # paramétrage de la nature ou tube identifié comme débouché
                    # par le trieur (code OPEN ou pas de champ MAT)
                    if order["zone_groupement_test"]:
                        codes_zones = ",".join(set([order["zone_groupement_test"] for order in sample.orders]))
                        if priority:
                            msg.tests = "{},STAT".format(codes_zones)
                        else:
                            msg.tests = "{}".format(codes_zones)
                    else:
                        msg.tests = ""
                else:
                    # La couleur présente dans le message de QUERY est
                    # incompatible avec la couleur définie dans le paramétrage
                    # de la nature.
                    msg.tests = "ERREUR"
            else:
                msg.tests = "{},{}".format(sample.specimen, tests)
                msg.material_code = sample.specimen
        if self.pvs_version_routage_zone and context["query"] and msg == "":
            return self.create(
                "RQ",
                typ="RQ",
                tube=context["query"][0].tube,
                tests="ERREUR"
            )
        else:
            return msg

    def decode_message(self, message):
        for m in message:
            if m.typ == "LA":
                worklist = Worklist(context="query")
                if self.pvs_version_routage_zone and m.material and len(m.material.split(" ")) == 4:
                    sample_color = self.COLOR_CODE.get(m.material.split(" ")[3])
                else:
                    sample_color = ""
                if m.tube:
                    worklist.get_or_create(
                        type="patient",
                        sample_id=m.tube,
                        sample_color=sample_color
                    )
            elif m.typ == "WP":
                # pas de résultat réceptionné en production
                # c'est la position qui devrait être récupérée
                # manque d'exemple
                if self.pvs_version_routage_zone:
                    rack = "{}-{}".format(m.rack, m.pos)
                else:
                    rack = m.pos
                worklist = Worklist(context="result")
                worklist.get_or_create(
                    sample_id=m.tube,
                    rack=rack,
                    type="patient",
                )
            else:
                raise NotImplementedError("Type {} inconnu".format(m.typ,))

        return worklist
